/*
 * Copyright 2015, Olivier MATZ <zer0@droids-corp.org>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef UCG_IRQ_H_
#define UCG_IRQ_H_

#include <stdint.h>

typedef uint32_t ucg_irqflags_t;

static inline uint32_t __ucg_get_primask(void)
{
	ucg_irqflags_t primask;

	__asm volatile(
		"mrs r0, primask\n"
		"strb r0, %0\n"
		: "=m" (primask)
		:
		: "r0");

	return primask;
}

static inline void __ucg_set_primask(ucg_irqflags_t primask)
{
	__asm volatile (
		"ldrb r0, %0\n"
		"msr primask,r0;\n"
		:
		: "m" (primask)
		: "r0");
}

static inline void ucg_irq_lock(void)
{
	__asm volatile ("cpsie i");
}

static inline void ucg_irq_unlock(void)
{
	__asm volatile ("cpsid i");
}

static inline ucg_irqflags_t ucg_irq_lock_save(void)
{
	ucg_irqflags_t flags;

	flags = __ucg_get_primask();
	ucg_irq_unlock();
	return flags;
}

static inline void ucg_irq_unlock_restore(ucg_irqflags_t flags)
{
	__ucg_set_primask(flags);
}

static inline int ucg_irq_locked(void)
{
	ucg_irqflags_t flags = __ucg_get_primask();

	return !!flags;
}

#endif /* UCG_IRQ_H_ */
