/*
 * Copyright 2015, Olivier MATZ <zer0@droids-corp.org>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

#include <stdio.h>
#include <ctype.h>

#include <ucg_irq.h>
#include <ucg_delay.h>

#include "uart.h"

static void ss_high(void)
{
	printf_P(PSTR("ss_high\r\n"));
	PORTB |= (1 << 2);
}

static void ss_low(void)
{
	printf_P(PSTR("ss_low\r\n"));
	PORTB &= (~(1 << 2));
}

static uint8_t spi_sendrecv(uint8_t tx)
{
	uint8_t rx;

	SPDR = tx;

	/* Wait for transmission complete */
	while ((SPSR & (1 << SPIF)) == 0)
		;

	ucg_delay_ms(100);
	rx = SPDR;

	printf_P(PSTR("sent 0x%2.2x, recvd 0x%2.2x '%c'\r\n"),
		tx, rx, isprint(rx) ? rx : '.');
	ucg_delay_ms(1000);

	return rx;
}

int main(void)
{
	uint8_t i;

	/* spi: SS (PB2), MOSI (PB3), SCK (PB5) */
	DDRB = (1 << 2) | (1 << 3) | (1 << 5);

	ss_high();

	/* blink led before start (unfortunatly it's on SCK, we can't
	 * use it during spi transfer) */
	for (i = 0; i < 3; i++) {
		PORTB |= (1 << 5);
		ucg_delay_ms(500);
		PORTB &= (~(1 << 5));
		ucg_delay_ms(500);
	}

	uart_init();
	fdevopen(std_send, std_recv);

	ucg_irq_unlock();

	printf_P(PSTR("hello\r\n"));

#if 0 /* test serial (echo) */
	{
		int16_t c;

		while (1) {
			c = std_recv(stdin);
			if (c > 0)
				std_send(c, stdout);
		}
	}
#endif

	/* remove power reduction on spi */
	PRR &= ~(1 << PRSPI);

	/* Enable SPI, Master, set clock rate fck/16 */
	SPCR = (1 << SPE) | (1 << MSTR) | (1 << SPR0);

	ss_low();
	ucg_delay_ms(1000);

	/* read 5 bytes at address 0x1000 */
	spi_sendrecv(0x03);
	spi_sendrecv(0x00);
	spi_sendrecv(0x10);
	spi_sendrecv(0x00);
	for (i = 0; i < 12; i++)
		spi_sendrecv(0x00); /* data 0 to 12 */
	ss_high();

	while (1);
	return 0;
}
