/*
 * Copyright (c) 2015, Olivier MATZ <zer0@droids-corp.org>
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>

#include <stdio.h>
#include <ctype.h>

#include <ucg_irq.h>
#include <ucg_cirbuf.h>
#include <ucg_uart.h>
#include <ucg_avr_uart.h>

#include "uart.h"

/* rx & tx buffers */
static char rx_buf[128];
static struct ucg_cirbuf rx_cirbuf;
static char tx_buf[128];
static struct ucg_cirbuf tx_cirbuf;
/* generic uart struct */
static struct ucg_uart main_uart;
/* avr-specific uart struct */
static struct ucg_avr_uart avr_uart_data = {
	.reg_udr = &UDR0,
	.reg_ucsra = &UCSR0A,
	.reg_ucsrb = &UCSR0B,
	.reg_ubrrl = &UBRR0L,
	.reg_ubrrh = &UBRR0H,
	.bit_udre = UDRE0,
	.bit_udrie = UDRIE0,
	.bit_rxen = RXEN0,
	.bit_txen = TXEN0,
	.bit_rxcie = RXCIE0,
};

/* send on stdout */
int std_send(char c, FILE *f)
{
	(void)f;
	ucg_uart_send(&main_uart, c);
	return 0;
}

/* recv on stdin */
int std_recv(FILE *f)
{
	int16_t c;

	(void)f;
	c = ucg_uart_recv_nowait(&main_uart);
	if (c < 0)
		return _FDEV_EOF;

	return c;
}

SIGNAL(USART_RX_vect)
{
	ucg_uart_recv_next_char(&main_uart);
}

SIGNAL(USART_UDRE_vect)
{
	ucg_uart_send_next_char(&main_uart);
}

int uart_init(void)
{
	int ret;
	struct ucg_uart_config conf;

	ret = ucg_uart_init(&main_uart, &avr_uart_ops, &avr_uart_data,
		&rx_cirbuf, rx_buf, sizeof(rx_buf),
		&tx_cirbuf, tx_buf, sizeof(tx_buf));
	if (ret < 0)
		return ret;
	ucg_uart_getconf(&main_uart, &conf);
	conf.baudrate = 57600;
	ret = ucg_uart_setconf(&main_uart, &conf);
	return ret;
}
