#
# Copyright 2015, Olivier MATZ <zer0@droids-corp.org>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the University of California, Berkeley nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

# variables that must be defined:
#  UCGINE: path to ucgine root
#  PROG: name of the binary
#  SRCS: list of source files (without path)
#  O: output path
#
# variable that can be defined:
#  CPPFLAGS: list of preprocessor flags
#  CFLAGS: list of compiler flags
#  LDFLAGS: list of linker flags
#  LDLIBS: list of libraries to link with
#  VPATH: paths where source can be located
#  prefix
#  exec_prefix
#  bindir

prefix ?= /usr/local
exec_prefix ?= $(prefix)
bindir ?= $(exec_prefix)/bin

include $(UCGINE)/mk/ucgine.common.mk
-include $(addprefix $(O)/,$(DEPS))
-include $(addprefix $(O)/,$(CMDS))
-include $(call file2cmd,$(O)/$(PROG))

# link a file
#  $1: sources (*.o)
#  $2: dst (binary)
link = $(CC) $(LDFLAGS) -o $(2) $(1) $(LDLIBS)

# print line to link a file
ifeq ($(V),1)
link_print_cmd = echo $(call protect_quote,$(call link,$1,$2))
else
link_print_cmd = echo "  PROG $(2)"
endif

ABS_OBJS = $(abspath $(addprefix $(O)/,$(OBJS)))

ifeq ($(D),1)
$(info $(SRCS))
$(info $(abspath $(SRCS)))
$(info $(ABS_OBJS))
endif

$(O)/$(PROG): $(ABS_OBJS) FORCE
	@[ -d $(dir $@) ] || mkdir -p $(dir $@)
	@$(call display_deps,$(ABS_OBJS),$@,$(call link,$(ABS_OBJS),$@),$^,$?)
	@$(if $(call check_deps,$@,$(call link,$(ABS_OBJS),$@),$^,$?),\
		$(call link_print_cmd,$(ABS_OBJS),$@) && \
		$(call link,$(ABS_OBJS),$@) && \
		$(call save_cmd,$(call link,$(ABS_OBJS),$@),$@) && \
		$(call create_empty_depfile,$(@)))

.SECONDEXPANSION:
$(O)/%.o: %.c $$(wildcard $$(dep_$$@)) FORCE
	@[ -d $(dir $@) ] || mkdir -p $(dir $@)
	@$(call display_deps,$<,$@,$(call compile,$<,$@),$^,$?)
	@$(if $(call check_deps,$@,$(call compile,$<,$@),$^,$?),\
		$(call compile_print_cmd,$<,$@) && \
		$(call compile,$<,$@) && \
		$(call save_cmd,$(call compile,$<,$@),$@) && \
		$(call fixdep,$@))

.SECONDEXPANSION:
#$(O)/%.o: %.s $$(wildcard $$(dep_$$@)) FORCE
$(O)/%.o: %.s FORCE
	@[ -d $(dir $@) ] || mkdir -p $(dir $@)
	@$(call display_deps,$<,$@,$(call compile,$<,$@),$^,$?)
	@$(if $(call check_deps,$@,$(call compile,$<,$@),$^,$?),\
		$(call compile_print_cmd,$<,$@) && \
		$(call compile,$<,$@) && \
		$(call save_cmd,$(call compile,$<,$@),$@) && \
		$(call fixdep,$@))

install install-target: $(DESTDIR)$(bindir)/$(PROG)

$(DESTDIR)$(bindir)/$(PROG): $(O)/$(PROG)
	install -D -m 775 $< $@

clean:
	rm -f $(addprefix $(O)/,$(OBJS) $(DEPS) $(CMDS) $(PROG))

.PHONY: install install-target clean FORCE
